'use strict'

const rp = require('request-promise');
const { jar } = require('request');
const Bluebird = require('bluebird');
const { CookieJar, MemoryCookieStore } = require('tough-cookie');
const cheerio = require('cheerio');
const fs = require('fs'); 
const _ = require('lodash');
const crypto = require('crypto');
const ora = require('ora');
const os = require('os');
const EventEmitter = require("events");
const Json2csvParser = require("json2csv").Parser;
const CONST = require('./constant');

const followersParser = new Json2csvParser();

class TwitterBot extends EventEmitter{
    constructor({ username, password, proxy, number, cli, session_path, csv, json, stream, id, event, timeout }){
        super()
        this._steam = stream || false;
        this._username = username;
        this._password = password;
        this._proxy = proxy;
        this._mainHost = 'https://twitter.com/';
        this._apiHostGraphQl = 'https://api.twitter.com/graphql/';
        this._apiHostV2 = 'https://api.twitter.com/2/';
        this._apiHostV1 = 'https://api.twitter.com/1.1/';
        this._cookieStore = new MemoryCookieStore();
        this._cookieJar = jar(this._cookieStore);
        this._number = number || 0;
        this._collector = {};
        this._json2csvParser = {};
        this._cursor = '';
        this._session = false;
        this._event = false || event;
        this._session_file_path = session_path || os.tmpdir();
        this._graphqlEndPoints = {
            'UserByScreenName': '',
        }
        this._cli = cli;
        this._csv = false || csv;
        this._json = false || json;
        this._timeout = 0 || timeout;
        this._spinner = cli ? ora('Twitter Scraper Started').start() : '';
    }
    
    get _csrftoken(){
        let cookie = this._extractCookie('ct0');
        if (!cookie){
            cookie = crypto.randomBytes(16).toString('hex');
            this._cookieJar.setCookie(`ct0=${cookie}; Expires=${new Date(Date.now()+21600000).toGMTString()}; Max-Age=21600; Domain=twitter.com; Path=/; Secure; HttpOnly; SameSite=Lax; hostOnly=false`, this._mainHost );
        }else{
            cookie = cookie.value
        }
        return cookie;
    }

    _extractCookie(key){
        const cookies = this._cookieJar.getCookies(this._mainHost);
        return _.find(cookies, { key }) || null;
    }

    // Main request
    _request({uri, headers, method, qs, json, body, form, timeout, simple, jar, gzip, br, session}){
        return new Promise( async (resolve, reject) => {
            try{
                let response = await rp({
                    uri: uri,
                    method,
                    ...(qs ? { qs } : {}),
                    ...(body ? { body } : {}),
                    ...(form ? { form } : {}),
                    'headers':{
                        'user-agent':'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.13; rv:69.0) Gecko/20100101 Firefox/69.0',
                        ...headers,
                        'accept-language':'en-US,en;q=0.5',
                        'te':'trailers',
                    },
                    ...(json ?{ json: true } : {}),
                    ...(gzip ? { gzip: true } : {}),
                    ...(br ? {brotliDecompress: decompressStream} : {}),
                    ...(simple ? { 'simple': false } : {} ),
                    ...(jar ? { 'jar': this._cookieJar} : { }),
                    'resolveWithFullResponse': true,
                    ...(this._proxy ? {proxy:`https://${this._proxy}/`} : {}),
                });
                if(session){
                    await this._renewSession();
                }
                if (this._timeout){
                    setTimeout(() => {
                        resolve(response);
                    },this._timeout)
                }else{
                    resolve(response);
                }
            } catch(error){
                if (error.name === 'StatusCodeError'){
                    reject(error.response.body)
                }else{
                    reject({errors:[{ message: error.message ? error.message : 'Request error'}]})
                }
            }
        })
    }

    // If there is no exisiting session then open twitter to grab all required headers
    _openMainPage(){
        return new Bluebird( async (resolve, reject) => {
            let request = {
                'method': 'GET',
                'uri': this._mainHost,
                'jar': true,
                'gzip':true,
                'session':true,
            }
            try{
                let response = await this._request(request);
                let $ = cheerio.load(response.body);
                this._authenticity_token = $('[name="authenticity_token"]')[0].attribs.value;
                resolve()
            } catch(error){
                reject(error)
            }
        })
    }

   // Creating and storing session for future usage 
    _createSession(){
        return new Promise( async (resolve, reject) => {
            if (!this._username || !this._password){
                return reject({errors:[{ message: 'In order for you to use this scraper you need to signin with your username and password'}]})
            }
            try{
                await this._openMainPage();

                let request = {
                    'uri': `https://twitter.com/sessions`,
                    'method': 'POST',
                    'form':{
                        'session[username_or_email]': this._username,
                        'session[password]': this._password,
                        'authenticity_token': this._authenticity_token,
                        'remember_me': 1,
                    },
                    'headers':{
                        'referer': 'https://twitter.com/login',
                    },
                    'json': true,
                    'gzip':true,
                    'simple': true,
                    'jar': true,
                    'session':true,
                }
                if (this._cli){
                    this._spinner.text = 'Trying to create session';
                }
                let response = await this._request(request);
                if (response.headers.location.indexOf('error')>-1){
                    return reject({errors:[{ message: `Can't signin to the account, please verify username and password`}]})
                }
                resolve()
            } catch(error){
                reject(error)
            }
        })
    }
    async _serializeCookieJar(){
        return await Bluebird.fromCallback(cb => this._cookieJar['_jar'].serialize(cb))
    }

    async _deserializeCookieJar(cookies) {
        this._cookieJar['_jar'] =  await Bluebird.fromCallback(cb => CookieJar.deserialize(cookies, this._cookieStore, cb));
        this._csrftoken;
    }

    async _loadExisitingSession(){
        try{
            await Promise.all([
                this._loadSessionFromFile(),
                this._getBearerToken()
            ]);
            await this._validateSession();
            this._session = true;
        } catch(error){
            try{
                await this._createSession();
                this._session = true;
                return Promise.resolve();
            }catch(error){
                if (this._cli){
                    this._spinner.stop()
                }
                return Promise.reject(error);
            }
        }
    }

    _loadSessionFromFile(){
        return new Promise(async (resolve,reject) => {
            try{
                let file_cookies = await Bluebird.fromCallback(cb => fs.readFile(`${this._session_file_path}/${this._username}.json`, {encoding:'utf-8'}, cb));
                resolve(await this._deserializeCookieJar(file_cookies));
            }catch(error){
                reject(error);
            }
        })
    }

    _validateSession(){
        return new Promise( async (resolve, reject) => {
            let request = {
                'uri': `${this._apiHostV2}guide.json`,
                'method': 'GET',
                'qs':{
                    'include_profile_interstitial_type': 0,
                    'include_blocking': 0,
                    'include_blocked_by': 0,
                    'include_followed_by': 0,
                    'include_want_retweets': 0,
                    'include_mute_edge': 0,
                    'include_can_dm': 0,
                    'include_can_media_tag': 0,
                    'skip_status': 1,
                    'cards_platform': 'Web-12',
                    'include_cards': 1,
                    'include_composer_source': 'false',
                    'include_ext_alt_text': 'false',
                    'include_reply_count': 1,
                    'tweet_mode': 'extended',
                    'include_entities': 'false',
                    'include_user_entities': 'false',
                    'include_ext_media_color': 'false',
                    'include_ext_media_availability': 'false',
                    'send_error_codes': 'false',
                    'count': 20,
                    'ext': 'mediaStats,highlightedLabel,cameraMoment',
                },
                'headers':{
                    'accept': '*/*',
                    'authorization': `Bearer ${this._bearerToken}`,
                    'x-twitter-auth-type': 'OAuth2Session',
                    'x-twitter-client-language': 'en',
                    'x-twitter-active-user': 'yes',
                    'x-csrf-token': this._csrftoken,
                    'origina': 'https://twitter.com',
                    'referer': 'https://twitter.com/'
                },
                'jar':true,
                'json': true,
                'session':true,
            }
            try{
                resolve(await this._request(request));
            } catch(error){
                reject(error)
            }
        })
    }
    
    // Renew session
    _renewSession(){
        return new Promise(async (resolve, reject) => {
            let newCookies = await this._serializeCookieJar();
            try{
                resolve(await Bluebird.fromCallback(cb => fs.writeFile(`${this._session_file_path}/${this._username}.json`, JSON.stringify(newCookies), {flag: 'w'} , cb)));
            } catch(error){
                reject(error)
            }
        })
    }

    // Get Bearer Token
    _getBearerToken(){
        return new Promise( async (resolve, reject) => {
            let request = {
                'method': 'GET',
                'uri': 'https://abs.twimg.com/responsive-web/web/main.a8f2d8608bc5a83e4.js',
                'headers':{
                    'accept':'*/*',
                    'origina': 'https://twitter.com',
                    'referer': 'https://twitter.com/'
                },
                'gzip':true,
                'session':false,
            }
            try{
                let response = await this._request(request);
                this._bearerToken = response.body.split('u="')[1].split('"')[0];
                this._graphqlEndPoints['UserByScreenName'] = response.body.split('r0on:')[1].split('"')[1];
                resolve()
            } catch(error){
                reject(error)
            }
        })
    }
    _setCsvParser(action, init_seacrch){
        if (!this._event && this._csv && CONST[`csv_${action}`]){
            return this._json2csvParser[`${action}_${init_seacrch}`] = new Json2csvParser({ fields: CONST[`csv_${action}`] });
        }
    }

    _input({ type, action, input, number }){
        return new Promise( async(resolve, reject) => {
            if (!this._session){
                try{
                    await this._loadExisitingSession();
                } catch(error){
                    if (this._event){
                        return this.emit('error', error);
                    }else{
                        return reject(error);
                    }
                }
            }

            if (!this._session){
                if(this._cli){
                    this._spinner.stop()
                }
                if (this._event){
                    return this.emit('error', {errors:[{ message: `In order to use this scraper you need to authorize your account`}]});
                }else{
                    return reject({errors:[{ message: `In order to use this scraper you need to authorize your account`}]});
                }
            }
            let init_seacrch = Date.now();
            this._collector[`${action}:${init_seacrch}:${input}`] = [];
            let result = '';
            this._number = number;
            this._setCsvParser(action, init_seacrch);
            switch(type){
                case 'scrape':
                    try{
                        switch(action){
                            case 'retweetedBy':
                                if (this._cli){
                                    this._spinner.text = 'Scraping retweetedBy data';
                                }
                                result = await this._retweetedBy(input, this._collector[`${action}:${init_seacrch}:${input}`]);
                                break;
                            case 'likedBy':
                                if (this._cli){
                                    this._spinner.text = 'Scraping likedBy data';
                                }
                                result = await this._likedBy(input, this._collector[`${action}:${init_seacrch}:${input}`]);
                                break;
                            case 'conversation':
                                if (this._cli){
                                    this._spinner.text = 'Scraping conversation data';
                                }
                                result = await this._conversation(input, this._collector[`${action}:${init_seacrch}:${input}`]);
                                break;
                            case 'followers':
                                result = await this._followers(input, this._collector[`${action}:${init_seacrch}:${input}`]);
                                break;
                            case 'tweets':
                                result = await this._tweets(input, this._collector[`${action}:${init_seacrch}:${input}`]);
                                break;
                            case 'favorites':
                                result = await this._favorites(input, this._collector[`${action}:${init_seacrch}:${input}`]);
                                break;
                            default:
                                if (this._event){
                                    return this.emit('error', {errors:[{ message: `Unknown action`}]});
                                }else{
                                    return reject({errors:[{ message: `Unknown action`}]});
                                }
                        }
                        // Removing collector storage and cleaning cursor
                        this._cursor = '';
                        delete this._collector[`${action}:${init_seacrch}:${input}`];

                        if (!this._event){
                            if (result.length>=1){
                                if (this._csv){
                                    this._csv_path = `${action}_${init_seacrch}_${input}.csv`;
                                    await Bluebird.fromCallback(cb => fs.writeFile(`${this._csv_path}`, this._json2csvParser[`${action}_${init_seacrch}`].parse(result), cb))
                                }
                                if (this._json){
                                    this._json_path = `${action}_${init_seacrch}_${input}.json`;
                                    await Bluebird.fromCallback(cb => fs.writeFile(`${this._json_path}`, JSON.stringify(result), cb))
                                }
                            }
                            if (this._cli){
                                this._spinner.stop();
                            }
                            return resolve({
                                collector: result,
                                ...(this._json_path ? { json: this._json_path } : { }),
                                ...(this._csv_path ? { csv: this._csv_path } : { }),
                                ...(result.length === 0 ? { message: `Nothing was scraped. Probably rate limit was reached, you can wait a little and try again and set higher timeout for example 3000`} : {})
                            })
                        }
                        this.emit('done')
                    } catch(error){
                        if (this._cli){
                            this._spinner.stop();
                        }
                        if (this._event){
                            this.emit('error', error);
                        }else{
                            reject(error)
                        }
                    }
                    break;
                default:
                    if (this._event){
                        this.emit('error', {errors:[{ message: `Wrong action`}]});
                    }else{
                        reject({errors:[{ message: `Wrong action`}]});
                    }
            }
        })
    }

    // Start TimeLine Requests
    _timeline(by, tweet_id, count, user_id){
        return new Promise( async (resolve, reject) => {
            let request = {
                'uri': `${this._apiHostV2}timeline/${by}.json`,
                'method': 'GET',
                'qs':{
                    'include_profile_interstitial_type': 0,
                    'include_blocking': 0,
                    'include_blocked_by': 0,
                    'include_followed_by': 0,
                    'include_want_retweets': 0,
                    'include_mute_edge': 0,
                    'include_can_dm': 0,
                    'include_can_media_tag': 0,
                    'skip_status': 1,
                    'cards_platform': 'Web-12',
                    'include_cards': 1,
                    'include_composer_source': 'false',
                    'include_ext_alt_text': 'false',
                    'include_reply_count': 1,
                    'tweet_mode': 'extended',
                    'include_entities': 'false',
                    'include_user_entities': 'false',
                    'include_ext_media_color': 'false',
                    'include_ext_media_availability': 'false',
                    'send_error_codes': 'false',
                    ...(user_id ? { userId: user_id } : { }),
                    ...(tweet_id ? { 'tweet_id': tweet_id } : { }),
                    'count': count,
                    'ext': 'mediaStats,highlightedLabel,cameraMoment',
                    ...(this._cursor ? { 'cursor': this._cursor } : {}),
                },
                'headers':{
                    'accept': '*/*',
                    'authorization': `Bearer ${this._bearerToken}`,
                    'x-twitter-auth-type': 'OAuth2Session',
                    'x-twitter-client-language': 'en',
                    'x-twitter-active-user': 'yes',
                    'x-csrf-token': this._csrftoken,
                    'origina': 'https://twitter.com',
                    'referer': 'https://twitter.com/'
                },
                'jar':true,
                'json': true,
                'session':true,
            }
            try{
                resolve(await this._request(request));
            } catch(error){
                if (this._cli){
                    this._spinner.stop();
                }
                reject(error)
            }
        })
    }
    _retweetedBy(tweet_id, collector){
        return new Promise( async (resolve, reject) => {
            try{
                let { body } = await this._timeline('retweeted_by', tweet_id, 100);
                for (let key in body.globalObjects.users){
                    if (collector.length>=this._number){
                        break;
                    }
                    if (this._event){
                        this.emit('data', body.globalObjects.users[key])
                        collector.push('');
                    }else{
                        collector.push(body.globalObjects.users[key]);
                    }
                }
                if (!this._event){
                    resolve(collector);
                }
            } catch(error){
                if (this._event){
                    this.emit('error', error)
                }else{
                    reject(error)
                }
            }
        })
    }

    _likedBy(tweet_id, collector){
        return new Promise( async (resolve, reject) => {
            try{
                let { body } = await this._timeline('liked_by', tweet_id, 100)
                for (let key in body.globalObjects.users){
                    if (collector.length>=this._number){
                        break;
                    }
                    if (this._event){
                        this.emit('data', body.globalObjects.users[key])
                        collector.push('');
                    }else{
                        collector.push(body.globalObjects.users[key]);
                    }
                }
                if (!this._event){
                    resolve(collector);
                }
            } catch(error){
                if (this._event){
                    this.emit('error', error)
                }else{
                    reject(error)
                }
            }
        })
    }

    _conversation(tweet_id, collector){
        return new Promise( async (resolve, reject) => {
            collectItems: while(true){
                if (collector.length>=this._number){
                    break;
                }
                try{
                    let { body } = await this._timeline(`conversation/${tweet_id}`, '', 20);
                    let len = body.timeline.instructions[0].addEntries.entries.length;
                    for (let key in body.globalObjects.tweets){
                        if (collector.length>=this._number){
                            break collectItems;
                        }
                        body.globalObjects.tweets[key].created_at_mls = new Date(body.globalObjects.tweets[key].created_at).getTime();
                        body.globalObjects.tweets[key].source = body.globalObjects.tweets[key].source.split('>')[1].split('<')[0];
                        if (this._event){
                            this.emit('data', body.globalObjects.tweets[key])
                            collector.push('');
                        }else{
                            collector.push(body.globalObjects.tweets[key]);
                        }
                    }
                    this._cursor = body.timeline.instructions[0].addEntries.entries[len-1].content.operation.cursor.value;
                } catch(error){
                    if (error instanceof TypeError){
                        error = `Nothing to scrape`
                    }
                    if (this._event){
                        this.emit('error', error);
                    }
                    break;
                }
            }
            if (!this._event){
                if (this._cli){
                    this._spinner.text = 'Sorting conversation data by date';
                }
                collector.sort((a,b) => {
                    return b.created_at_mls - a.created_at_mls;
                });
                resolve(collector);
            }
        })
    }

    _tweets(username, collector){
        return new Promise( async (resolve, reject) => {
            if (this._cli){
                this._spinner.text = 'Gathering user data';
            }

            try{
                await this._UserByScreenName(username)
            }catch(error){
                return reject(error);
            }

            if (this._cli){
                this._spinner.text = 'Scraping user tweets';
            }
            collectItems: while(true){
                if (collector.length>=this._number){
                    break;
                }
                try{
                    let { body } = await this._timeline(`profile/${this._userFromSearch.rest_id}`, '', 20, this._userFromSearch.rest_id);
                    let len = body.timeline.instructions[0].addEntries.entries.length;
                    for (let key in body.globalObjects.tweets){
                        if (collector.length>=this._number){
                            break collectItems;
                        }
                        body.globalObjects.tweets[key].created_at_mls = new Date(body.globalObjects.tweets[key].created_at).getTime();
                        body.globalObjects.tweets[key].source = body.globalObjects.tweets[key].source.split('>')[1].split('<')[0];
                        if (this._event){
                            this.emit('data', body.globalObjects.tweets[key])
                            collector.push('');
                        }else{
                            collector.push(body.globalObjects.tweets[key]);
                        }
                    }
                    this._cursor = body.timeline.instructions[0].addEntries.entries[len-1].content.operation.cursor.value;
                } catch(error){
                    if (error instanceof TypeError){
                        error = `Nothing to scrape`
                    }
                    if (this._event){
                        this.emit('error', error);
                    }
                    break;
                }
            }
            if (!this.event){
                if (this._cli){
                    this._spinner.text = 'Sorting user tweets by date';
                }
                collector.sort((a,b) => {
                    return b.created_at_mls - a.created_at_mls;
                });
                resolve(collector);
            }
        })
    }

    _favorites(username, collector){
        return new Promise( async (resolve, reject) => {
            if (this._cli){
                this._spinner.text = 'Gathering user data';
            }

            try{
                await this._UserByScreenName(username)
            }catch(error){
                reject(error);
            }
            
            if (this._cli){
                this._spinner.text = 'Scraping user favorites';
            }
            collectItems: while(true){
                if (collector.length>=this._number){
                    break;
                }
                try{
                    let { body } = await this._timeline(`favorites/${this._userFromSearch.rest_id}`, '', 20);
                    let len = body.timeline.instructions[0].addEntries.entries.length;
                    for (let key in body.globalObjects.tweets){
                        if (collector.length>=this._number){
                            break collectItems;
                        }
                        body.globalObjects.tweets[key].created_at_mls = new Date(body.globalObjects.tweets[key].created_at).getTime();
                        body.globalObjects.tweets[key].source = body.globalObjects.tweets[key].source.split('>')[1].split('<')[0];
                        if (this._event){
                            this.emit('data', body.globalObjects.tweets[key])
                            collector.push('');
                        }else{
                            collector.push(body.globalObjects.tweets[key]);
                        }
                    }
                    this._cursor = body.timeline.instructions[0].addEntries.entries[len-1].content.operation.cursor.value;
                } catch(error){
                    if (error instanceof TypeError){
                        error = `Nothing to scrape`
                    }
                    if (this._event){
                        this.emit('error', error);
                    }
                    break;
                }
            }
            if (!this._event){
                if (this._cli){
                    this._spinner.text = 'Sorting user favorites by date';
                }
                collector.sort((a,b) => {
                    return b.created_at_mls - a.created_at_mls;
                });
                resolve(collector);
            }
        })
    }
    // End TimeLine Requests

    _followers(username, collector){
        return new Promise( async (resolve, reject) => {
            if (this._cli){
                this._spinner.text = 'Gathering user data';
            }
            try{
                await this._UserByScreenName(username)
            }catch(error){
                return reject(error);
            }
            if (this._cli){
                this._spinner.text = 'Scraping user followers data';
            }
            collectItems: while(true){
                let request = {
                    'uri': `${this._apiHostV1}followers/list.json`,
                    'method': 'GET',
                    'qs':{
                        'include_profile_interstitial_type': '1',
                        'include_blocking': '1',
                        'include_blocked_by': '1',
                        'include_followed_by': '1',
                        'include_want_retweets': '1',
                        'include_mute_edge': '1',
                        'include_can_dm': '1',
                        'include_can_media_tag': '1',
                        'skip_status': '1',
                        'cursor': '-1',
                        'user_id': this._userFromSearch.rest_id,
                        ...(this._cursor ? {cursor: this._cursor} : {}),
                        'count': '100'
                    },
                    'headers':{
                        'accept': '*/*',
                        'authorization': `Bearer ${this._bearerToken}`,
                        'x-twitter-auth-type': 'OAuth2Session',
                        'x-twitter-client-language': 'en',
                        'x-twitter-active-user': 'yes',
                        'x-csrf-token': this._csrftoken,
                        'origina': 'https://twitter.com',
                        'referer': 'https://twitter.com/'
                    },
                    'jar':true,
                    'json': true,
                    'session':true,
                }
                if (collector.length>=this._number){
                    break;
                }
                try{
                    let { body } = await this._request(request);
                    for (let i =0; i<body.users.length; i++){
                        if (collector.length>=this._number){
                            if (this._steam){
                                //this.push( null );
                            }
                            break collectItems;
                        }
                        body.users[i].created_at_mls = new Date(body.users[i].created_at).getTime();
                        if (this._event){
                            this.emit('data', body.users[i])
                            collector.push('');
                        }else{
                            collector.push(body.users[i]);
                        }
                    }
                    this._cursor = body.next_cursor_str;
                } catch(error){
                    if (error instanceof TypeError){
                        error = `Nothing to scrape`
                    }
                    if (this._event){
                        this.emit('error', error);
                    }
                    break;
                }
            }
            if (!this._event){
                if (this._cli){
                    this._spinner.text = 'Sorting user followers by following date';
                }
                collector.sort((a,b) => {
                    return b.created_at_mls - a.created_at_mls;
                });
                resolve(collector);
            }
        })
    }


    // Start GraphQL Endpoints
    _graphql(username, query){
        return new Promise( async (resolve, reject) => {
            let request = {
                'uri': `${this._apiHostGraphQl}${query}`,
                'method': 'GET',
                'qs':{
                    'variables' :JSON.stringify({"screen_name": username,"withHighlightedLabel":true}),
                },
                'headers':{
                    'accept': '*/*',
                    'authorization': `Bearer ${this._bearerToken}`,
                    'x-twitter-auth-type': 'OAuth2Session',
                    'x-twitter-client-language': 'en',
                    'x-twitter-active-user': 'yes',
                    'x-csrf-token': this._csrftoken,
                    'origina': 'https://twitter.com',
                    'referer': 'https://twitter.com/'
                },
                'jar':true,
                'json': true,
                'session':true,
            }
            try{
                resolve(await this._request(request));
            } catch(error){
                reject(error)
            }
        })
    }

    _UserByScreenName(username){
        return new Promise( async (resolve, reject) => {
            try {
                let response = await this._graphql(username, `${this._graphqlEndPoints['UserByScreenName']}/UserByScreenName`);

                if (!response.body.data.user){
                    return reject({errors:[{ message: 'Not Found'}]})
                }
                if (response.body.data.user.legacy.protected){
                    return reject({errors:[{ message: 'Profile is private'}]})
                }
                if (response.body.data.user.legacy.blocked_by){
                    return reject({errors:[{ message: 'Profile is blocked'}]})
                }
                this._userFromSearch = response.body.data.user;
                resolve(response.body.data.user);
            }catch(error){
                reject(error)
            }
        })
    }
    //End GraphQL Endpoints
}

module.exports = TwitterBot;